@@ -27,19 +27,19 @@ class InviteeRow extends PureComponent<Props> {
         <td>{invitee.email}</td>
         <td>{invitee.name}</td>
         <td className="text-right">
-          <button className="btn btn-inverse btn-mini" onClick={this.copyToClipboard}>
+          <button className="btn btn-inverse btn-small" onClick={this.copyToClipboard}>
             <textarea
               readOnly={true}
               value={invitee.url}
               style={{ position: 'absolute', right: -1000 }}
               ref={this.copyUrlRef}
             />
-            <i className="fa fa-clipboard" /> Copy Invite
+            Copy Invite
           </button>
           &nbsp;
         </td>
         <td>
-          <button className="btn btn-danger btn-mini" onClick={() => revokeInvite(invitee.code)}>
+          <button className="btn btn-danger btn-small" onClick={() => revokeInvite(invitee.code)}>
             <i className="fa fa-remove" />
           </button>
         </td>
