@@ -5,20 +5,24 @@ import React, { PureComponent } from 'react';
 import { config } from 'app/core/config';
 
 // Components
-import { PieChart } from '@grafana/ui';
+import { PieChart, getFieldDisplayValues } from '@grafana/ui';
 
 // Types
 import { PieChartOptions } from './types';
-import { PanelProps } from '@grafana/ui/src/types';
-import { getSingleStatValues } from '../singlestat2/SingleStatPanel';
+import { PanelProps } from '@grafana/ui';
 
 interface Props extends PanelProps<PieChartOptions> {}
 
 export class PieChartPanel extends PureComponent<Props> {
   render() {
-    const { width, height, options } = this.props;
+    const { width, height, options, data, replaceVariables } = this.props;
 
-    const values = getSingleStatValues(this.props);
+    const values = getFieldDisplayValues({
+      fieldOptions: options.fieldOptions,
+      data: data.series,
+      theme: config.theme,
+      replaceVariables: replaceVariables,
+    }).map(v => v.display);
 
     return (
       <PieChart
